/*
     Copyright (C) 2006  Georges Da Costa

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>

#include <sys/types.h>

#define __USE_XOPEN /* strptime */
#include <time.h>

#include "gosd.h"

typedef struct cal_t {
  time_t current;
  win_t *win;
  unsigned int delta_jours;
  unsigned int f1;
  unsigned int f2;
  unsigned int f3;
  unsigned int c1;
  unsigned int c2;
} cal_t;

#include "toolbox.h"

void calendar(win_t *w, void *data) {
  int i;
  int level=1;
  char buffer[12];

  int current_month = 0;
  int first_day = 0;
  int current_pos=0; 

  time_t today;
  int today_day;
  int today_year;

  unsigned int color;

  struct tm *tm;

  cal_t *cal = (cal_t*)data;

  time(&today);
  tm=localtime(&today);
  today_day = tm->tm_yday;
  today_year = tm->tm_year;

  tm = localtime(&cal->current);

  strftime(buffer, 10, "%B", tm);
  gosd_print(w, 0, 0, cal->f3, cal->c1, buffer);

  strftime(buffer, 10, "%Y", tm);
  gosd_print(w, 155, 0, cal->f2, cal->c1, buffer);

  sprintf(buffer, "< O >");
  gosd_print(w, 155, 17, cal->f1, cal->c1, buffer);
  
  for(i = 0; i<7; i++) {
    char *names[7] = {"Su", "Mo", "Tu", "We", "Th", "Fr",
		      "Sa"};
    gosd_print(w, i*30, 18+20, cal->f1, cal->c1, names[(i+cal->delta_jours)%7]);
  }


  sprintf(buffer, "%d %d %d", 1, tm->tm_mon+1,
	  1900+tm->tm_year);
  strptime(buffer, "%d %m %Y", tm);

  first_day = tm->tm_yday+1;
  current_month = tm->tm_mon;
  
  for(i = 0; i<31; i++) {

    sprintf(buffer, "%d %d", i+first_day,
	    1900+tm->tm_year);
    strptime(buffer, "%j %Y", tm);
    
    if(tm->tm_mon != current_month)
      break;

    current_pos = (tm->tm_wday+7-cal->delta_jours)%7;
    sprintf(buffer,"%d",i+1);

    if(today_day == tm->tm_yday &&
       today_year == tm->tm_year)
      color = cal->c2;
    else
      color = cal->c1;

    gosd_print(w, 15+current_pos*30-gosd_text_width(w, cal->f1,buffer), level*18+40,
	       cal->f1, color, buffer);

    if(current_pos == 6) level++;
  }
}

void prev_month(unsigned int a, unsigned int b,unsigned int c, void* arg) {
  struct tm *tm;
  char buffer[16];

  tm = localtime(&((cal_t*)arg)->current);

  if(tm->tm_mon == 0) 
    sprintf(buffer, "%d %d %d", 1, 12, 1900+tm->tm_year-1);
  else
    sprintf(buffer, "%d %d %d", 1, tm->tm_mon+1-1, 1900+tm->tm_year);
  strptime(buffer, "%d %m %Y", tm);
  ((cal_t*)arg)->current = mktime(tm);
}

void next_month(unsigned int a, unsigned int b,unsigned int c, void* arg) {
  struct tm *tm;
  char buffer[16];

  tm = localtime(&((cal_t*)arg)->current);

  if(tm->tm_mon == 11) 
    sprintf(buffer, "%d %d %d", 1, 1, 1900+tm->tm_year+1);
  else
    sprintf(buffer, "%d %d %d", 1, tm->tm_mon+2, 1900+tm->tm_year);
  strptime(buffer, "%d %m %Y", tm);

  ((cal_t*)arg)->current = mktime(tm);
}

void current_month(unsigned int a, unsigned int b,unsigned int c, void* arg) {
  time(&((cal_t*)arg)->current);
}

void *new_calendar(win_t *win, unsigned int f1, unsigned int f2, unsigned int f3,
		   unsigned int c1, unsigned int c2, unsigned int delta) {
  cal_t *res;

  res=malloc(sizeof(cal_t));

  time(&res->current);
  res->delta_jours=delta;
  res->win=win;
  res->f1=f1;
  res->f2=f2;
  res->f3=f3;
  res->c1=c1;
  res->c2=c2;

  return (void*) res;
}
